from .client import Client

class HttpClient(Client):
    ''' Butter MAS HTTP client API '''

    def __init__(self, ip, port=5555, protocol='http'):
        """Initialize Butter MAS HTTP client
                
        Args:
            ip (str): robot IP
            port (int, optional): robot port. Defaults to 5555.
            protocol (str, optional): communication protocol. Defaults to "http".
        """
        super().__init__(ip, port, protocol)

if __name__ == "__main__":
    client = HttpClient('localhost')

    print(client.getAvailableHandlers().json())


from .packet_factory import PacketFactory

class PacketBuilder():
    ''' Builds a command packet using the builder design pattern '''

    def __init__(self, ip, port, protocol="http"):
        """Initialize builder
        
        Args:
            ip (str): robot IP
            port (int): robot port
            protocol (str, optional): communication protocol. defaults to "http".
        """
        self.ip = ip
        self.port = port

        packetFactory = PacketFactory()
        self.packet = packetFactory.getPacketClass(protocol)

        self.cmd = None
        self.args = list()
        self.params = list()
        self.keys = dict()

    def addCommand(self, command):
        """Add command
        
        Args:
            command (str): command
        
        Returns:
            PacketBuilder: self
        """
        self.cmd = command

        return self

    def addArgument(self, argument):
        """Add argument
        
        Args:
            argument (str): argument
        
        Returns:
            PacketBuilder: self
        """
        self.args.append(argument)

        return self
    
    def addArguments(self, *arguments):
        """Add arguments
        
        Returns:
            PacketBuilder: self
        """
        if arguments:
            for argument in arguments:
                self.args.append(argument)

        return self

    def addParameter(self, parameter):
        """Add parameter
        
        Args:
            parameter (str): parameter
        
        Returns:
            PacketBuilder: self
        """
        self.params.append(self._formatParameter(parameter))

        return self

    def addParameters(self, *parameters):
        """Add parameters
        
        Returns:
            PacketBuilder: self
        """
        if parameters:
            for parameter in parameters:
                self.params.append(self._formatParameter(parameter))

        return self

    def addKeyValuePair(self, key, value):
        """Add key value pair
        
        Args:
            key (str): attribute key
            value (str): attribute value
        
        Returns:
            PacketBuilder: self
        """
        self.keys[key] = value

        return self

    def _formatParameter(self, parameter):
        """Formats parameter properly
        
        Args:
            parameter (str): parameter
        
        Returns:
            str: formated parameter
        """
        if not parameter.startswith('--'):
            if  parameter.startswith('-'):
                parameter = '-' + parameter
            else:
                parameter = '--' + parameter

        return parameter

    def build(self):
        """Builds the packet
        
        Returns:
            Packet: data packet
        """
        if self.cmd == None: 
            return None

        query = "%s?" % self.cmd

        if self.args: 
            query = "%s%s&" % (query, '&'.join(self.args))

        if self.params: 
            params = list(map(self._formatParameter, self.params))
            query = "%s%s&" % (query, '&'.join(params))

        if self.keys:
            keys = list(map(lambda x: "x=%s" % self.keys[x], self.keys))
            query = "%s%s" % (query, '&'.join(keys))

        uri = '/'.join(['cmd', 'json'])
        uri = "%s/%s" % (uri, query.strip('&'))

        return self.packet(self.ip, self.port, uri)



from abc import ABC, abstractmethod

class Packet(ABC):
    ''' Represents an abstract data packet '''

    def __init__(self, ip, port, query):
        """Initialize packet
        
        Args:
            ip (str): robot IP
            port (int, optional): robot port. Defaults to 5555.
            query (str): packet payload
        """
        self.ip = ip
        self.port = port
        self.query = query

    @abstractmethod
    def send(self, timeout=None):
        """Send packet abstract methud
        
        Args:
            timeout (int, optional): packet timeout. defaults to None.
        """
        pass

    def __eq__(self, other):
        return isinstance(other, Packet) and self.ip == other.ip and self.port == other.port and self.query == other.query



from .utils.packet_builder import PacketBuilder

class Client():
    ''' Butter MAS client API '''

    def __init__(self, ip, port=5555, protocol='http'):
        """Initialize client
        
        Args:
            ip (str): robot IP
            port (int, optional): robot port. Defaults to 5555.
            protocol (str, optional): communication protocol. Defaults to "http".
        """
        self.ip = ip
        self.port = port
        self.protocol = protocol

    def getAvailableHandlers(self):
        """Get available robot handlers
        
        Returns:
            Response: response containing all the available robot handlers
        """
        packet = PacketBuilder(self.ip, self.port, self.protocol).addCommand('list').build()

        return packet.send()

    def getAvailableAnimations(self, reload=False):
        """Get available (loaded) robot animations        
        
        Args:
            reload (bool, optional): reload all animations. Defaults to False.
        
        Returns:
            Response: response containing all the available (loaded) robot animations
        """
        builder = PacketBuilder(self.ip, self.port, self.protocol).addCommand('animate')

        if reload:
            builder.addParameter('reload')

        packet = builder.addParameter('list').build()

        return packet.send()

    def getAvailableSounds(self, reload=False):
        """Get available (loaded) robot sound assets        
        
        Args:
            reload (bool, optional): reload all sound assets. Defaults to False.
        
        Returns:
            Response: response containing all the available (loaded) robot sound assets
        """
        builder = PacketBuilder(self.ip, self.port, self.protocol).addCommand('audio')

        if reload:
            builder.addParameter('reload')

        packet = builder.addParameter('list').build()

        return packet.send()

    def getAvailableMotorRegisters(self, motorName, readableOnly=False):
        """Get all available motor registers
        
        Args:
            motorName (str): motor name (as configured on the configurator)
            readableOnly (bool, optional): get readable registers only. Defaults to False.
        
        Returns:
            Response: response containing all the available motor registers
        """
        packet = PacketBuilder(self.ip, self.port, self.protocol).addCommand('get').addParameter('list') \
                    .addKeyValuePair('readableOnly', readableOnly).build()

        return packet.send()

    def getMotorRegister(self, motorName, registerName):
        """Get motor register value
        
        Args:
            motorName (str): motor name (as configured on the configurator)
            registerName (str): motor register name
        
        Returns:
            Response: response containing register value
        """
        packet = PacketBuilder(self.ip, self.port, self.protocol).addCommand('get') \
                    .addArguments(motorName, registerName).build()

        return packet.send()

    def setMotorRegister(self, motorName, registerName, value):
        """Get motor register value
        
        Args:
            motorName (str): motor name (as configured on the configurator)
            registerName (str): motor register name
            value (str): register value
        
        Returns:
            Response: response containing execution result
        """
        packet = PacketBuilder(self.ip, self.port, self.protocol).addCommand('set') \
                    .addArguments(motorName, registerName, value).build()        

        return packet.send()

    def playAnimation(self, animationName):
        """Play animation on the robot
        
        Args:
            animationName (str): animation name
        
        Returns:
            Response: response containing execution result
        """
        packet = PacketBuilder(self.ip, self.port, self.protocol).addCommand('animate').addArgument(animationName).build()        

        return packet.send()

    def pauseAnimation(self):
        """Pause currently playing animation (if available) on the robot
        
        Returns:
            Response: response containing execution result
        """
        packet = PacketBuilder(self.ip, self.port, self.protocol).addCommand('animate').addParameter('pause').build()        

        return packet.send()

    def resumeAnimation(self):
        """Resume currently paused animation (if available) on the robot
        
        Returns:
            Response: response containing execution result
        """
        packet = PacketBuilder(self.ip, self.port, self.protocol).addCommand('animate').addParameter('resume').build()        

        return packet.send()

    def stopAnimation(self):
        """Stop currently playing animation (if available) on the robot
        
        Returns:
            Response: response containing execution result
        """
        packet = PacketBuilder(self.ip, self.port, self.protocol).addCommand('animate').addParameter('stop').build()        

        return packet.send() 

    def playAudio(self, fileName):
        """Play audio on the robot
        
        Args:
            fileName (str): audio asset name
        
        Returns:
            Response: response containing execution result
        """
        packet = PacketBuilder(self.ip, self.port, self.protocol).addCommand('audio').addArgument(fileName).build()        

        return packet.send()

    def pauseAudio(self):
        """Pause current audio playback (if available) on the robot
        
        Returns:
            Response: response containing execution result
        """
        packet = PacketBuilder(self.ip, self.port, self.protocol).addCommand('audio').addParameter('pause').build()        

        return packet.send()

    def resumeAudio(self):
        """Resume currently paused audio playback (if available) on the robot
        
        Returns:
            Response: response containing execution result
        """
        packet = PacketBuilder(self.ip, self.port, self.protocol).addCommand('audio').addParameter('resume').build()        

        return packet.send()

    def stopAudio(self):
        """Stop current audio playback (if available) on the robot
        
        Returns:
            Response: response containing execution result
        """
        packet = PacketBuilder(self.ip, self.port, self.protocol).addCommand('audio').addParameter('stop').build()        

        return packet.send()



from .client import Client

class UdpClient(Client):
    ''' Butter MAS UDP client API '''

    def __init__(self, ip, port=5000, protocol='udp'):
        """Initialize Butter MAS UDP client
                
        Args:
            ip (str): robot IP
            port (int, optional): robot port. Defaults to 5000.
            protocol (str, optional): communication protocol. Defaults to "udp".
        """
        super().__init__(ip, port, protocol)

if __name__ == "__main__":
    client = UdpClient('localhost')

    print(client.getAvailableHandlers())


import unittest
from butter.mas.utils.packet_builder import PacketBuilder
from butter.mas.utils.packet_factory import PacketFactory

class TestPacketBuilderMethods(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.builder = None

    def setUp(self):
        self.builder = PacketBuilder('localhost', 5555)

    def testAddCommand(self):
        self.builder.addCommand('get')
        self.assertEqual(self.builder.cmd, 'get')

    def testAddArgument(self):
        self.builder.addArgument('base')
        self.assertEqual(self.builder.args, ['base'])
    
    def testAddArguments(self):
        self.builder.addArguments('base', 'top')
        self.assertEqual(self.builder.args, ['base', 'top'])

    def testAddParameter(self):
        self.builder.addParameter('play')
        self.builder.addParameter('-pause')
        self.builder.addParameter('--stop')
        self.assertEqual(self.builder.params, ['--play', '--pause', '--stop'])

    def testAddParameters(self):
        self.builder.addParameters('play', '-pause', '--stop')
        self.assertEqual(self.builder.params, ['--play', '--pause', '--stop'])

    def testAddKeyValuePair(self):
        self.builder.addKeyValuePair('speed', '5')
        self.assertDictEqual(self.builder.keys, {'speed': '5'})

    def testBuild(self):
        packet = self.builder.addCommand('get').addParameter('help').build()
        packetFactory = PacketFactory()
        self.assertEqual(packet, packetFactory.getPacket('localhost', 5555, 'cmd/json/get?--help'))

if __name__ == '__main__':
    unittest.main()


import socket
from .packet import Packet
from .general_utils import print_error

class UdpPacket(Packet):
    ''' Represents a udp data packet '''

    def __init__(self, ip, port, query):
        """Initialize packet
        
        Args:
            ip (str): robot IP
            port (int): robot port
            query (str): packet payload
        """
        super().__init__(ip, port, query)
        self.udpSocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.bufferSize  = 1024

    def send(self, timeout=5):
        """Send packet
        
        Args:
            timeout (int, optional): packet timeout. defaults to 5.
        
        Returns:
            Response: response containing the response
        """
        response = None

        try:
            self.udpSocket.sendto(bytes(self.query, "utf-8"), (self.ip, self.port))
            # response = self.udpSocket.recvfrom(self.bufferSize)
        except socket.herror as host_error:
            print_error('Warning: we have encountered in a host error.\n%s\n' % host_error)
            response = self._generateEmptyResponse('host')
        except socket.gaierror as address_error:
            print_error('Warning: we have encountered in a address error.\n%s\n' % address_error)
            response = self._generateEmptyResponse('address')
        except socket.timeout as timeout_error:
            print_error('Warning: request timeout have been reached.\n%s\n' % timeout_error)
            response = self._generateEmptyResponse('timeout')
        except (OSError | InterruptedError) as error:
            print_error('Warning: request failed.\n%s\n' % error)
            response = self._generateEmptyResponse()

        return response

    @staticmethod
    def _generateEmptyResponse(errorType=b'unknown'):
        """Generates empty response packet
        
        Args:
            errorType (bytes, optional): error type. defaults to b'unknown'.
        
        Returns:
            Response: error response
        """
        error = '{ "exception": "Request resolved with an %s error" }' % errorType

        return (error, None)

    def __eq__(self, other):
        return isinstance(other, UdpPacket) and self.ip == other.ip and self.port == other.port and self.query == other.query


from .client_http import HttpClient
from .client_udp import UdpClient

class ClientFactory:
    ''' Client factory for diffrent types of protocols '''

    def getClient(self, ip, port=None, protocol="http"):
        """Creates new client
        
        Args:
            ip (str): robot IP
            port (int, optional): robot port. Defaults to None.
            protocol (str, optional): communication protocol. Defaults to "http".
        
        Returns:
            Client: requested client
        """
        if (protocol == "http"):
            return HttpClient(ip) if port is None else HttpClient(ip, port)
        elif (protocol == "udp"):
            return UdpClient(ip) if port is None else UdpClient(ip, port)
        else:
            return None

    def getClientClass(self, protocol="http"):
        """Get client class
        
        Args:
            protocol (str, optional): communication protocol. Defaults to "http".
        
        Returns:
            Client: client class
        """
        if (protocol == "http"):
            return HttpClient
        elif (protocol == "udp"):
            return UdpClient
        else:
            return None



from .packet_http import HttpPacket
from .packet_udp import UdpPacket

class PacketFactory:
    ''' Packet factory for diffrent types of protocols '''

    def getPacket(self, ip, port, query, protocol="http"):
        """Creates new packet
        
        Arguments:
            ip (str): robot IP
            port (int): robot port
            query (str): packet payload
            protocol (str, optional): communication protocol. defaults to "http".
        
        Returns:
            Packet: data packet
        """
        if (protocol == "http"):
            return HttpPacket(ip, port, query)
        elif (protocol == "udp"):
            return UdpPacket(ip, port, query)
        else:
            return None

    def getPacketClass(self, protocol="http"):
        """Get packet class
        
        Args:
            protocol (str, optional): communication protocol. defaults to "http".
        
        Returns:
            Packet: data packet
        """
        if (protocol == "http"):
            return HttpPacket
        elif (protocol == "udp"):
            return UdpPacket
        else:
            return None



import unittest
from butter.mas.utils.packet_factory import PacketFactory

class TestUdpPacketMethods(unittest.TestCase):

    def testSend(self):
        packetFactory = PacketFactory()
        packet = packetFactory.getPacket('localhost', 5555, 'cmd/json/help', protocol='udp')
        self.assertIsNotNone(packet.send())

if __name__ == '__main__':
    unittest.main()


import requests
from .packet import Packet
from .general_utils import print_error

class HttpPacket(Packet):
    ''' Represents a http data packet '''

    def __init__(self, ip, port, query):
        """Initialize packet
        
        Args:
            ip (str): robot IP
            port (int): robot port
            query (str): packet payload
        """
        super().__init__(ip, port, query)

    def send(self, timeout=5):
        """Send packet
        
        Args:
            timeout (int, optional): packet timeout. defaults to 5.
        
        Returns:
            Response: response containing the response
        """
        response = None

        try:
            response = requests.get('http://%s:%s/%s' % (self.ip, self.port, self.query), timeout=timeout)
        except requests.exceptions.HTTPError as http_error:
            print_error('Warning: we have encountered in a http error.\n%s\n' % http_error)
            response = self._generateEmptyResponse('http')
        except requests.exceptions.Timeout as timeout_error:
            print_error('Warning: request timeout have been reached.\n%s\n' % timeout_error)
            response = self._generateEmptyResponse('timeout')
        except requests.exceptions.ConnectionError as connection_error:
            print_error('Warning: we have encountered in a connection error.\n%s\n' % connection_error)
            response = self._generateEmptyResponse('connection')
        except requests.exceptions.RequestException as error:
            print_error('Warning: request failed.\n%s\n' % error)
            response = self._generateEmptyResponse()

        return response

    @staticmethod
    def _generateEmptyResponse(errorType=b'unknown'):
        """Generates empty response packet
        
        Args:
            errorType (bytes, optional): error type. defaults to b'unknown'.
        
        Returns:
            Response: error response
        """
        response = requests.Response()

        response._content = b'{ "exception": "Request resolved with an %s error" }' % errorType.encode("ascii")
        response.code = "expired"
        response.error_type = "expired"
        response.status_code = 400

        return response

    def __eq__(self, other):
        return isinstance(other, HttpPacket) and self.ip == other.ip and self.port == other.port and self.query == other.query


# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

with open('README.md', encoding="utf8") as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()

setup(
    name='butter.mas-api',
    version='0.8.5',
    description='Python HTTP Client API for Butter MAS platform',
    long_description=readme,
    author='Benny Megidish',
    author_email='bennymegk@gmail.com',
    url='https://github.com/benymeg/Butter.MAS.PythonAPI',
    license=license,
    packages=find_packages(exclude=('tests', 'docs'))
)

''' how to deploy '''
# python setup.py sdist bdist_wheel
# python -m twine upload --repository-url https://test.pypi.org/legacy/ dist/*
# pip install --index-url https://test.pypi.org/simple/ --no-deps butter.mas-api

# python -m twine upload dist/*
# pip install butter.mas-api
